@@ -4,11 +4,11 @@ mod card;
 mod compilation_error;
 mod compile_options;
 mod lane;
-mod program;
+mod module;
 
 pub mod card_description;
 
-mod compiled_lane;
+mod lane_ir;
 #[cfg(test)]
 mod tests;
 
@@ -19,40 +19,42 @@ use crate::{
     prelude::TraceEntry,
     Instruction, NodeId, VariableId,
 };
-use std::fmt::Debug;
+use std::convert::TryFrom;
 use std::mem;
-use std::{cell::RefCell, convert::TryFrom};
+use std::{borrow::Cow, fmt::Debug};
 use std::{convert::TryInto, str::FromStr};
 
 pub use card::*;
 pub use compilation_error::*;
 pub use compile_options::*;
 pub use lane::*;
-pub use program::*;
+pub use module::*;
 
-use self::compiled_lane::CompiledLane;
+use self::lane_ir::LaneIr;
 
 pub type CompilationResult<T> = Result<T, CompilationError>;
 
 /// Intermediate representation of a Cao-Lang program.
 ///
 /// Execution will begin with the first Lane
-pub(crate) type CaoIr<'a> = &'a [CompiledLane];
-pub(crate) type NameSpace = smallvec::SmallVec<[String; 8]>;
+pub(crate) type LaneSlice<'a> = &'a [LaneIr];
+pub(crate) type NameSpace = smallvec::SmallVec<[Box<str>; 8]>;
+pub(crate) type Imports = std::collections::HashMap<String, String>;
 
 pub struct Compiler<'a> {
     options: CompileOptions,
     program: CaoCompiledProgram,
-    next_var: RefCell<VariableId>,
+    next_var: VariableId,
 
     /// maps lanes to their metadata
     jump_table: KeyMap<LaneMeta>,
 
-    current_namespace: NameSpace,
+    current_namespace: Cow<'a, NameSpace>,
+    current_imports: Cow<'a, Imports>,
     locals: Box<arrayvec::ArrayVec<Local<'a>, 255>>,
     scope_depth: i32,
     current_card: i32,
-    current_lane: String,
+    current_lane: Box<str>,
 }
 
 #[derive(Debug, Clone, Copy)]
@@ -79,7 +81,7 @@ pub fn compile(
         .map_err(|err| CompilationError::with_loc(err, LaneNode::default(), 0))?;
 
     let mut compiler = Compiler::new();
-    compiler.compile(&compilation_unit, Some(options))
+    compiler.compile(&compilation_unit, options)
 }
 
 impl<'a> Default for Compiler<'a> {
@@ -93,27 +95,23 @@ impl<'a> Compiler<'a> {
         Compiler {
             options: Default::default(),
             program: CaoCompiledProgram::default(),
-            next_var: RefCell::new(VariableId(0)),
+            next_var: VariableId(0),
             jump_table: Default::default(),
             current_namespace: Default::default(),
             locals: Default::default(),
             scope_depth: 0,
             current_card: -1,
-            current_lane: "".to_owned(),
+            current_lane: "".into(),
+            current_imports: Default::default(),
         }
     }
 
     pub fn compile(
         &mut self,
-        compilation_unit: CaoIr<'a>,
-        compile_options: impl Into<Option<CompileOptions>>,
+        compilation_unit: LaneSlice<'a>,
+        compile_options: CompileOptions,
     ) -> CompilationResult<CaoCompiledProgram> {
-        self.options = compile_options.into().unwrap_or_default();
-        // minimize the surface of the generic function
-        self._compile(compilation_unit)
-    }
-
-    fn _compile(&mut self, compilation_unit: CaoIr<'a>) -> CompilationResult<CaoCompiledProgram> {
+        self.options = compile_options;
         if compilation_unit.is_empty() {
             return Err(CompilationError::with_loc(
                 CompilationErrorPayload::EmptyProgram,
@@ -121,12 +119,12 @@ impl<'a> Compiler<'a> {
                 0,
             ));
         }
-        // initialize
         self.program = CaoCompiledProgram::default();
-        self.next_var = RefCell::new(VariableId(0));
+        self.next_var = VariableId(0);
         self.compile_stage_1(compilation_unit)?;
         self.compile_stage_2(compilation_unit)?;
 
+        self.current_imports = Default::default();
         Ok(mem::take(&mut self.program))
     }
 
@@ -140,7 +138,7 @@ impl<'a> Compiler<'a> {
 
     /// build the jump table and consume the lane names
     /// also reserve memory for the program labels
-    fn compile_stage_1(&mut self, compilation_unit: CaoIr) -> CompilationResult<()> {
+    fn compile_stage_1(&mut self, compilation_unit: LaneSlice) -> CompilationResult<()> {
         // check if len fits in 16 bits
         let _: u16 = match compilation_unit.len().try_into() {
             Ok(i) => i,
@@ -169,21 +167,21 @@ impl<'a> Compiler<'a> {
         Ok(())
     }
 
-    fn add_lane(&mut self, nodekey: Handle, n: &CompiledLane) -> CompilationResult<()> {
+    fn add_lane(&mut self, nodekey: Handle, n: &LaneIr) -> CompilationResult<()> {
         let metadata = LaneMeta {
             hash_key: nodekey,
             arity: n.arguments.len() as u32,
         };
-        let namekey = Handle::from_str(n.name.as_str()).expect("Failed to hash lane name");
+        let namekey = Handle::from_str(n.name.as_ref()).expect("Failed to hash lane name");
         if self.jump_table.contains(namekey) {
-            return Err(self.error(CompilationErrorPayload::DuplicateName(n.name.clone())));
+            return Err(self.error(CompilationErrorPayload::DuplicateName(n.name.to_string())));
         }
         self.jump_table.insert(namekey, metadata).unwrap();
         Ok(())
     }
 
     /// consume lane cards and build the bytecode
-    fn compile_stage_2(&mut self, compilation_unit: CaoIr<'a>) -> CompilationResult<()> {
+    fn compile_stage_2(&mut self, compilation_unit: LaneSlice<'a>) -> CompilationResult<()> {
         let mut lanes = compilation_unit.iter().enumerate();
 
         if let Some((il, main_lane)) = lanes.next() {
@@ -266,17 +264,19 @@ impl<'a> Compiler<'a> {
     fn process_lane(
         &mut self,
         il: usize,
-        CompiledLane {
+        LaneIr {
             name,
             arguments,
             cards,
             namespace,
-        }: &'a CompiledLane,
+            imports,
+        }: &'a LaneIr,
         instruction_offset: i32,
     ) -> CompilationResult<()> {
         self.current_lane = name.clone();
         self.current_card = -1;
-        self.current_namespace = namespace.clone();
+        self.current_namespace = Cow::Borrowed(namespace);
+        self.current_imports = Cow::Borrowed(imports);
 
         // check if len fits in 16 bits
         let _len: u16 = match cards.len().try_into() {
@@ -305,7 +305,7 @@ impl<'a> Compiler<'a> {
         then: impl FnOnce(&mut Self) -> CompilationResult<()>,
     ) -> CompilationResult<()> {
         type Pos = i32;
-        assert!(
+        debug_assert!(
             matches!(
                 skip_instr,
                 Instruction::GotoIfTrue | Instruction::GotoIfFalse
@@ -327,18 +327,35 @@ impl<'a> Compiler<'a> {
     }
 
     fn encode_jump(&mut self, lane: &LaneNode) -> CompilationResult<()> {
+        // attempt to look up the function by name
         let mut to = self.jump_table.get(Handle::from(lane));
-        let mut i = self.current_namespace.len();
-        while i > 0 && to.is_none() {
+        if to.is_none() {
             // attempt to look up the function in the current namespace
-            //
-            // TODO: create handle from parts instead of building the string..
-            let mut name = self.current_namespace[0..i].join(".");
-            name.push('.');
-            name.push_str(lane.0.as_str());
-            let handle = Handle::from(name.as_str());
+
+            // current_namespace.join('.').push('.').push_str(lane.0)
+            let handle = Handle::from_bytes_iter(
+                self.current_namespace
+                    .iter()
+                    .flat_map(|x| [x.as_bytes(), ".".as_bytes()])
+                    .chain(std::iter::once(lane.0.as_bytes())),
+            );
             to = self.jump_table.get(handle);
-            i -= 1;
+        }
+        if to.is_none() {
+            // attempt to look up function in the imports
+            if let Some((_, alias)) = self
+                .current_imports
+                .iter()
+                .find(|(import, _)| *import == &lane.0)
+            {
+                let handle = Handle::from_bytes_iter(
+                    self.current_namespace
+                        .iter()
+                        .flat_map(|x| [x.as_bytes(), ".".as_bytes()])
+                        .chain(std::iter::once(alias.as_bytes())),
+                );
+                to = self.jump_table.get(handle);
+            }
         }
         let to = to.ok_or_else(|| {
             self.error(CompilationErrorPayload::InvalidJump {
@@ -453,7 +470,7 @@ impl<'a> Compiler<'a> {
                 write_to_vec(index, &mut self.program.bytecode);
             }
             Card::SetGlobalVar(variable) => {
-                let mut next_var = self.next_var.borrow_mut();
+                let next_var = &mut self.next_var;
                 if variable.0.is_empty() {
                     return Err(self.error(CompilationErrorPayload::EmptyVariable));
                 }
@@ -545,7 +562,7 @@ impl<'a> Compiler<'a> {
         let scope = self.resolve_var(variable.0.as_str())?;
         if scope < 0 {
             // global
-            let mut next_var = self.next_var.borrow_mut();
+            let next_var = &mut self.next_var;
             let varhash = Handle::from_bytes(variable.0.as_bytes());
             let id = self
                 .program
